/*-
 * Copyright (C) @BABOLO  2001 .. 2010 http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C) @BABOLO  2001 .. 2010 http://www.babolo.ru/\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: ispmsactive.c,v 1.5 2010/10/25 01:14:15 babolo Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <sys/event.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sysexits.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <fcntl.h>
#include <stdio.h>
#include <time.h>
#include <err.h>
#define DEFAULTWATCHFILENAME "ispmsactivecontrol"
int v_flag;

void usage(int ex)
{   printf( "ispmsactive " VERS "  " DATE " @BABOLO http://www.links.ru/~babolo/\n"
            "ispmsactive [-d watchfile] [-t timeout]\n"
            "     watchfile is ${CONFDIR}/" DEFAULTWATCHFILENAME " if not specifyed\n"
          );
#ifdef WATCHFILE
    printf( "     and " WATCHFILE "/" DEFAULTWATCHFILENAME " as a default\n"
          );
#endif
    exit(ex);
}

int main(int argc, char **argv)
{   int c, i;
    char *watchfile;
    int kq;
    struct kevent reg, event;
    int fd;
    struct timespec *timeout, noll;
    struct stat sf, sd;

    v_flag = 0;
    timeout = NULL;
    watchfile = NULL;
    bzero(&noll, sizeof(noll));
    while ((c = getopt(argc, argv, "d:t:v?")) > 0)
        switch (c)
        {case 'd': watchfile = optarg;                 break;
         case 't': if  ((timeout = malloc(sizeof(*timeout))) == NULL)              err(EX_OSERR, NULL);
                   bzero(timeout, sizeof(*timeout));
                   timeout->tv_sec = strtol(optarg, NULL, 0);
                                                      break;
         case 'v': v_flag++;                          break;
         case '?': usage(EX_OK);
         default:  usage(EX_USAGE);
        }
    argc -= optind;
    argv += optind;
    if  (watchfile == NULL && getenv("CONFDIR"))
        if  (asprintf(&watchfile, "%s/" DEFAULTWATCHFILENAME, getenv("CONFDIR")) < 0)
                                                                             err(EX_UNAVAILABLE, NULL);
#ifdef WATCHFILE
    if  (watchfile == NULL) watchfile = WATCHFILE
#else
    if  (watchfile == NULL)                                                            usage(EX_USAGE);
#endif
    if  ((kq = kqueue()) < 0)                                                      err(EX_OSERR, NULL);
    if  ((fd = open(watchfile, O_WRONLY | O_CREAT | O_TRUNC, 0666)) < 0)           err(EX_OSERR, NULL);
    EV_SET(&reg, fd, EVFILT_VNODE, EV_ADD, NOTE_DELETE, 0, NULL);
    i = kevent(kq, &reg, 1, &event, 1, &noll);
    if  (i == 0)
    {   i = stat(watchfile, &sf);
        fstat(fd, &sd);
        if  (v_flag > 2)
            fprintf( stderr, "%s stat = %d\n%d %d fstat\n%d %d stat\n"
                   , watchfile, i
                   , sd.st_dev, sd.st_ino
                   , sf.st_dev, sf.st_ino
                   );
        if  (  i == 0
            && sf.st_dev == sd.st_dev
            && sf.st_ino == sd.st_ino
            )
        {   if  (v_flag > 2) fprintf( stderr, "Wait for event now\n");
            i = kevent(kq, &reg, 1, &event, 1, timeout);
            if  (v_flag > 2) fprintf( stderr, "YES\n");
            close(fd);
    }   }
    if  (v_flag) fprintf(stderr, "kevent = %d\n", i);
    exit(0);
}
